-- holder for some lib functions
local cur_version=13
if _G["HED_LIB"] and hedlib.ver>=cur_version then 
	-- print ("Hed_lib "..cur_version.." is already")
	else 

hedlib = _G["HED_LIB"] or CreateFrame("Frame","HED_LIB",UIParent)
hedlib.ver=cur_version

--print("Hedd_lib version "..cur_version)

hedlib.dummy = function() end

hedlib.round = function (n)
	return floor(n+0.5)
end

hedlib.isPlayer = function(unit)
	if(unit == 'player' or unit == 'vehicle' or unit == 'pet') then
		return true
	else
		return nil
	end
end

hedlib.isEnemy = function(unit)
	if (UnitIsEnemy("player", unit) or UnitCanAttack("player", unit)) then
		return true
	else
		return nil
	end
end

hedlib.siValue = function(val)
	if val==0 then
		return
	elseif val >= 10000000 or val <= -10000000 then 
        return string.format("%.1fm", val / 1000000) 
    elseif val >= 1000000 or val <= -1000000 then
        return string.format("%.2fm", val / 1000000) 
    elseif val >= 100000 or val <= -100000 then
        return string.format("%.0fk", val / 1000) 
    elseif val >= 1000 or val <= -1000 then
        return string.format("%.1fk", val / 1000) 
    else
        return ceil(val)
    end
end

hedlib.hex = function(r, g, b)
	if(type(r) == 'table') then
		if(r.r) then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
	end
	return ('|cff%02x%02x%02x'):format(r * 255, g * 255, b * 255)
end

hedlib.fontsize = function(height)
	local FRAME_SIZE,FONT_SIZE = 1,0.9
	local size = FONT_SIZE * (height / FRAME_SIZE)
	size = (size>9) and size or 9
	return(hedlib.round(size))
end

hedlib.CreateBD = function(f,border,color, bgcolor)
	color = color or {1,0,0,1}
	border = border or 5
	local backdrop_tab = { 
    bgFile = hedlib.backdrop_texture, 
    edgeFile = hedlib.backdrop_edge_texture,
    tile = false,
    tileSize = 0, 
    edgeSize = border, 
    insets = { 
      left = border, 
      right = border, 
      top = border, 
      bottom = border,
    },
  	}
	f.bd = f.bd or CreateFrame("Frame", nil, f)
    f.bd:SetFrameLevel(0)
    f.bd:SetPoint("TOPLEFT",-border,border)
    f.bd:SetPoint("BOTTOMRIGHT",border,-border)
    f.bd:SetBackdrop(backdrop_tab)
	bgcolor = bgcolor or {0,0,0,0}
    f.bd:SetBackdropColor(unpack(bgcolor))
   	f.bd:SetBackdropBorderColor(unpack(color))
	return f.bd
end

hedlib.CreateBG=function(f,border,coltex,color)
	f.bg = f.bg or f:CreateTexture(nil, "BACKGROUND")
	border=border or 0
	if (border==0) then
		f.bg:SetAllPoints(f)
	else
		f.bg:SetPoint("TOPLEFT", -border, border)
		f.bg:SetPoint("BOTTOMRIGHT", border, -border)
	end
	coltex = coltex or {0,0,0}
	if type(coltex) == "string" then
		f.bg:SetTexture(coltex)
		if color then
			f.bg:SetVertexColor(unpack(color))
		end
	else	
		f.bg:SetTexture(unpack(coltex))
	end
	return f.bg
end

hedlib.CreateOverlay=function(f,border,coltex,color)
	f.overlay = f.overlay or f:CreateTexture(nil, "BACKGROUND")
	border=border or 0
	if (border==0) then
		f.overlay:SetAllPoints(f)
	else
		f.overlay:SetPoint("TOPLEFT", -border, border)
		f.overlay:SetPoint("BOTTOMRIGHT", border, -border)
	end
	coltex = coltex or {0,0,0}
	if type(coltex) == "string" then
		f.overlay:SetTexture(coltex)
		if color then
			f.overlay:SetVertexColor(unpack(color))
		end
	else	
		f.overlay:SetTexture(unpack(coltex))
	end
	return f.overlay
end

hedlib.CreateFont = function(f, font, size, outline)
	if size then 
		if size<1 then 
			size = hedlib.fontsize(f:GetHeight())+size
		end
	else
		size = hedlib.fontsize(f:GetHeight())
	end
	
	local fs = f:CreateFontString(nil, "OVERLAY")
	fs:SetFont(font, size, outline)
	fs:SetShadowColor(0,0,0,1)
	return fs
end  

hedlib.UpdateFS=function(f, size, justify)
    local fo,si,fl=f:GetFont()
    f:SetFont(fo, size, fl)
    f:SetShadowColor(0, 0, 0, 0)
    if(justify) then f:SetJustifyH(justify) end
    return f
end

hedlib.ColorGradient = function(perc, ...)
	if perc >= 1 then
		local r, g, b = select(select('#', ...) - 2, ...)
		return r, g, b
	elseif perc <= 0 then
		local r, g, b = ...
		return r, g, b
	end
	
	local num = select('#', ...) / 3

	local segment, relperc = math.modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)

	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

hedlib.executes = function()
	local playerClass, englishClass = UnitClass("player")
	if (englishClass=="WARRIOR" or englishClass=="HUNTER" or englishClass=="PALADIN") then
		return 20
	end
	return 0
end

local flashframe = nil
local flashsec = 0
hedlib.flash = function(sec,color)
	color = color or {1, 0, 0, 0.55}
	flashsec=sec
	flashframe = _G["HedFlash"] or CreateFrame("Frame", "HedFlash", UIParent)
	flashframe:SetFrameStrata("BACKGROUND")
	flashframe:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",})
	flashframe:SetAllPoints(UIParent)
	flashframe.elapsed = 0
	flashframe:SetScript("OnShow", function(self)
--		self.elapsed = 0
		self:SetAlpha(0)
	end)
	
	flashframe:SetScript("OnUpdate", function(self, elapsed)
		elapsed = self.elapsed + elapsed
		if elapsed >= flashsec then
			self:Hide()
			self:SetAlpha(0)
			self:SetScript("OnUpdate",nil)
			return
		end
		local alpha = elapsed % 0.4
		if elapsed > 0.2 then
			alpha = 0.4 - alpha
		end
		self:SetAlpha(alpha * 5)
		self.elapsed = elapsed
	end)
--	flashframe:Hide()

	flashframe:SetBackdropColor(unpack(color))
	flashframe:Show()
end

--[[ Creating info text ]]
local textframe = CreateFrame("Frame", nil, UIParent)
textframe:SetScript("OnUpdate", FadingFrame_OnUpdate)
textframe.fadeInTime = 0 -- 0.5
textframe.fadeOutTime = 0.5 -- 2
textframe.holdTime = 2 -- 3
textframe:Hide()
hedlib.textframe=textframe

local text = textframe:CreateFontString("HInfoText", "OVERLAY")
text:SetFont("Fonts\\FRIZQT__.TTF", 32, "THICKOUTLINE")
text:SetPoint("CENTER", UIParent, "CENTER",0,350)
text:SetTextColor(0.41, 0.8, 0.94)
hedlib.textframe.text=text

local icon=textframe:CreateTexture(nil, 'OVERLAY')
icon:SetHeight(30)
icon:SetWidth(30)
icon:SetPoint("RIGHT",text,"LEFT")
hedlib.textframe.icon=icon

hedlib.Warning = function(t,i,color)
	if i then
		color = color or {1,0,0}
	else
		color = color or {0.41, 0.8, 0.94}
	end
	hedlib.textframe.text:SetTextColor(unpack(color))
    hedlib.textframe.text:SetText(t)

	if (i) then
		hedlib.textframe.icon:SetTexture(i)
		hedlib.textframe.icon:Show()
	else
		hedlib.textframe.icon:Hide()
	end
	if hedlib.textframe:IsShown() then 
		hedlib.textframe.startTime=GetTime()
	else
	    FadingFrame_Show(hedlib.textframe)
	end
end

hedlib.TargetingYou = function(u)
	local uguid=UnitGUID(u.."target")
	local myguid=UnitGUID("vehicle") or UnitGUID("player")
	if (uguid and uguid==myguid) then
		return true
	else
		return nil
	end
end

hedlib.ClearTable = function(table)
	for k in pairs( table ) do
		table[k] = nil
	end
end

hedlib.tremall = function(tab)
   for k,v in pairs(tab) do
      table.remove(tab, k)
   end
end

hedlib.tremovebyval = function(tab, val)
   for k,v in pairs(tab) do
     if(v==val) then
       table.remove(tab, k)
       return true
     end
   end
   return false
end

hedlib.taddnewval = function(tab, val)
   for k,v in pairs(tab) do
     if(v==val) then
       return false
     end
   end
   table.insert(tab, val)  
   return true
end

local tf,dot

hedlib.tofloat = function(text,position)
	if text then
		text=string.gsub(text,",","%.")
		if position=="end" then 
			tf = string.match(text,"%d+%.?%d*$")
			tf = tf or 0
			return tonumber(tf)
		end
		if position=="start" then
			tf = string.match(text,"^%d+%.?%d*")
			tf = tf or 0
			return tonumber(tf)
		end
		tf = string.match(text,"%d+%.?%d*")
		tf = tf or 0
		return tonumber(tf)
	end
	return 0
end

hedlib.IsTextInTooltip = function(tt, txt)
	local regions = { tt:GetRegions() }
	for i, region in ipairs(regions) do
		if region and region:GetObjectType() == "FontString" then
			local c_txt=region:GetText()
            if c_txt and string.find(c_txt,txt) then
				return true,c_txt
			end
        end	
	end
	return nil
end

hedlib.classcolor = RAID_CLASS_COLORS[select(2,UnitClass'player')]
hedlib.classcolor = {hedlib.classcolor.r,hedlib.classcolor.g,hedlib.classcolor.b}
hedlib.addbox= function(name, toppoint,bottompoint,color,border,bcolor)
	if color == "CLASS" then
		color = hedlib.classcolor
	end
	if bcolor == "CLASS" then
		bcolor = hedlib.classcolor
	end
	
	local box = _G["HED_"..name] or CreateFrame("Frame","HED_"..name)
	box:SetParent(UIParent)
	box:SetPoint(unpack(toppoint))
	box:SetPoint(unpack(bottompoint))

	box.t = box:CreateTexture(nil, "BACKGROUND")
	box.t:SetAllPoints(box)
	box.t:SetTexture(unpack(color))
		
	if border then
		box:SetBackdrop({
			edgeFile = "Interface/Buttons/WHITE8X8",
			edgeSize = border,
			insets = { 
				left = border, 
				right = border, 
				top = border, 
				bottom = border,
			},
		})
		box:SetBackdropBorderColor(unpack(bcolor))
	end

	box:SetFrameStrata("BACKGROUND")
	box:Show()
end

local copy = {}
hedlib.shallowCopy = function(original)
    copy = {}
    for key, value in pairs(original) do
        copy[key] = value
    end
    return copy
end

hedlib.shallowCompare = function(original,old)
	local same=true
    for key, value in pairs(original) do
		if old[key] and old[key]==value then
			same=true
		else
			same=false
			return false
		end
    end
    return same
end

hedlib.stringifyTable = function(t)
    local entries = {}
    for k, v in pairs(t) do
        -- if we find a nested table, convert that recursively
        if type(v) == 'table' then
            v = hedlib.stringifyTable(v)
        else
            v = tostring(v)
        end
        k = tostring(k)
 
        -- add another entry to our stringified table
        entries[#entries + 1] = ("%s: %s\n"):format(k, v)
    end
 
    -- the memory location of the table
    local id = tostring(t):sub(8)
 
    return ("{%s}@%s"):format(table.concat(entries, ' = '), id)
end

hedlib.print = function(var)
if var then return var end
return "nil"
end

local num
hedlib.largest = function(numbers)
	num=numbers[1]
	for index,number in ipairs(numbers) do
		if num<number then num=number end
	end
	return num
end

hedlib.GetMax = function(numbers)
	num=numbers[1]
	for index,number in ipairs(numbers) do
		if num<number then num=number end
	end
	return num
end

hedlib.GetMin = function(numbers)
	num=numbers[1]
	for index,number in ipairs(numbers) do
		if num<number then num=number end
	end
	return num
end

hedlib.itemClasses = { GetAuctionItemClasses() }
--[["Weapon"
"Armor"
"Container"
"Consumable"
"Glyph"
"Trade Goods"
"Recipe"
"Gem"
"Miscallaneous"
"Quest"
"Battle Pets"]]--

--Taken from LibDispellable-1.0 sorry!
hedlib.enrage={}
for _, id in ipairs({
	4146, 8599, 12880, 15061, 15716, 16791, 18499, 18501, 19451, 19812,
	22428, 23128, 23257, 23342, 24689, 26041, 26051, 28371, 29340, 30485,
	31540, 31915, 32714, 33958, 34392, 34670, 37605, 37648, 37975, 38046,
	38166, 38664, 39031, 39575, 40076, 40601, 41254, 41364, 41447, 42705,
	42745, 43139, 43292, 43664, 47399, 48138, 48142, 48193, 48702, 49029,
	50420, 50636, 51170, 51513, 52071, 52262, 52309, 52461, 52470, 52537,
	53361, 54356, 54427, 54475, 54781, 55285, 55462, 56646, 56729, 56769,
	57733, 58942, 59465, 59694, 59697, 59707, 59828, 60075, 60177, 60430,
	61369, 63147, 63227, 66092, 66759, 67233, 68541, 69052, 70371, 72143,
	72203, 75998, 76100, 76487, 76816, 76862, 77238, 78722, 78943, 79420,
	80084, 80158, 80467, 81173, 81706, 81772, 82033, 82759, 86736, 90045,
	91668, 101109, 101110, 102134, 102989, 106925, 108169, 109889, 111220,
	111418, 115006, 115430, 115639, 116863, 116958, 117837, 118139, 118507,
	119629, 120093, 123914, 123936, 124019, 124172, 124309, 124840, 124976,
	125738, 125864, 126075, 126254, 126370, 126410, 127823, 127955, 128006,
	128231, 128248, 128809, 129016, 129874, 130196, 130202, 131150, 132710,
	134983, 135524, 135548, 135569, 135698, 137334, 140108, 141663, 142760,
	145554, 145692, 145974, 148295, 148852, 150759, 151553, 151965, 153909,
	154017, 154543, 155198, 155208, 155620, 156314, 157346, 158304, 158337,
	158456, 159479, 159748, 161601, 163121, 163483, 164257, 164324, 164811,
	164835, 165512, 168620, 172360, 172781, 173238, 173950, 174427, 175192,
	175337, 175463, 175586, 175743, 176023, 176048, 176214, 176396, 177152,
	178658,
}) do hedlib.enrage[id] = true end


end